@@ -11,6 +11,7 @@ warnings.simplefilter(action='ignore', category=Warning)
 warnings.simplefilter(action='ignore', category=DeprecationWarning)
 warnings.simplefilter(action='ignore', category=FutureWarning)
 warnings.simplefilter(action='ignore', category=RuntimeWarning)
+import subprocess
 import random
 random.seed(0)
 import numpy as np
@@ -38,6 +39,7 @@ def convert(
     onnx_graph: Optional[onnx.ModelProto] = None,
     output_folder_path: Optional[str] = 'saved_model',
     output_signaturedefs: Optional[bool] = False,
+    not_use_onnxsim: Optional[bool] = False,
     batch_size: Optional[int] = None,
     keep_ncw_or_nchw_or_ncdhw_input_names: Optional[List[str]] = None,
     replace_argmax_to_reducemax_and_indicies_is_int64: Optional[bool] = False,
@@ -46,6 +48,8 @@ def convert(
     replace_acos_to_pseudo_acos: Optional[bool] = False,
     replace_leakyrelu_to_pseudo_leakyrelu: Optional[bool] = False,
     replace_power_to_pseudo_power: Optional[bool] = False,
+    replace_gathernd_to_pseudo_gathernd: Optional[bool] = False,
+    replace_neg_to_pseudo_neg: Optional[bool] = False,
     param_replacement_file: Optional[str] = '',
     mvn_epsilon: Optional[float] = 0.0000000001,
     non_verbose: Optional[bool] = False,
@@ -72,6 +76,10 @@ def convert(
         to other model formats. However, this can significantly reduce the speed\n
         of model conversion and significant increase the size of the model.
 
+    not_use_onnxsim: Optional[bool]
+        No optimization by onnx-simplifier is performed.\n
+        If this option is used, the probability of a conversion error is very high.
+
     batch_size: Optional[int]
         Fixes the dynamic batch size to the specified numeric batch size.\n
         A value of 1 or more must be specified.
@@ -107,6 +115,12 @@ def convert(
     replace_power_to_pseudo_power: Optional[bool]
         Replace Power with a pseudo Power.
 
+    replace_gathernd_to_pseudo_gathernd: Optional[bool]
+        Replace GatherND with a pseudo GatherND.
+
+    replace_neg_to_pseudo_neg: Optional[bool]
+        Replace Neg with a pseudo Neg.
+
     mvn_epsilon: Optional[float]
         For MeanVarianceNormalization.\n
         The number to be added to the variance to avoid division by zero when normalizing the value.\n
@@ -114,7 +128,7 @@ def convert(
         Default: 0.0000000001
 
     param_replacement_file: Optional[str]
-        Parameter replacement path. (.json)
+        Parameter replacement file path. (.json)
 
     non_verbose: Optional[bool]
         Do not show all information logs. Only error logs are displayed.\n
@@ -192,10 +206,42 @@ def convert(
             )
             sys.exit(1)
 
+    # onnx-simplifier
+    # To fully optimize the model, run onnxsim three times in a row.
+    # Due to unstable script execution of onnxsim in v0.4.8,
+    # I have no choice but to use subprocesses that we do not want to use.
+    if not not_use_onnxsim:
+        try:
+            if not non_verbose:
+                print('')
+                print(f'{Color.REVERCE}Model optimizing started{Color.RESET}', '=' * 60)
+            for _ in range(3):
+                result = subprocess.check_output(
+                    [
+                        'onnxsim',
+                        f'{input_onnx_file_path}',
+                        f'{input_onnx_file_path}'
+                    ],
+                    stderr=subprocess.PIPE
+                ).decode('utf-8')
+                if not non_verbose:
+                    print(result)
+            if not non_verbose:
+                print(f'{Color.GREEN}Model optimizing complete!{Color.RESET}')
+        except Exception as e:
+            if not non_verbose:
+                print(
+                    f'{Color.YELLOW}WARNING:{Color.RESET} '+
+                    'Failed to optimize the onnx file.'
+                )
+                tracetxt = traceback.format_exc().splitlines()[-1]
+                print(f'{Color.YELLOW}WARNING:{Color.RESET} {tracetxt}')
+
     # Loading Graphs
     # onnx_graph If specified, onnx_graph is processed first
     if not onnx_graph:
         onnx_graph = onnx.load(input_onnx_file_path)
+
     graph = gs.import_onnx(onnx_graph)
 
     if not non_verbose:
@@ -216,6 +262,8 @@ def convert(
         'replace_acos_to_pseudo_acos': replace_acos_to_pseudo_acos,
         'replace_leakyrelu_to_pseudo_leakyrelu': replace_leakyrelu_to_pseudo_leakyrelu,
         'replace_power_to_pseudo_power': replace_power_to_pseudo_power,
+        'replace_gathernd_to_pseudo_gathernd': replace_gathernd_to_pseudo_gathernd,
+        'replace_neg_to_pseudo_neg': replace_neg_to_pseudo_neg,
         'replacement_parameters': replacement_parameters,
         'mvn_epsilon': mvn_epsilon,
     }
@@ -374,6 +422,14 @@ def main():
             'to other model formats. However, this can significantly reduce the speed \n' +
             'of model conversion and significant increase the size of the model.'
     )
+    parser.add_argument(
+        '-nuo',
+        '--not_use_onnxsim',
+        action='store_true',
+        help=\
+            'No optimization by onnx-simplifier is performed. \n' +
+            'If this option is used, the probability of a conversion error is very high.'
+    )
     parser.add_argument(
         '-b',
         '--batch_size',
@@ -437,6 +493,18 @@ def main():
         action='store_true',
         help='Replace Power with a pseudo Power.'
     )
+    parser.add_argument(
+        '-rgn',
+        '--replace_gathernd_to_pseudo_gathernd',
+        action='store_true',
+        help='Replace GatherND with a pseudo GatherND.'
+    )
+    parser.add_argument(
+        '-rng',
+        '--replace_neg_to_pseudo_neg',
+        action='store_true',
+        help='Replace Neg with a pseudo Neg.'
+    )
     parser.add_argument(
         '-me',
         '--mvn_epsilon',
@@ -453,7 +521,7 @@ def main():
         '--param_replacement_file',
         type=str,
         default='',
-        help='Parameter replacement path. (.json)'
+        help='Parameter replacement file path. (.json)'
     )
     parser.add_argument(
         '-n',
@@ -468,6 +536,7 @@ def main():
         input_onnx_file_path=args.input_onnx_file_path,
         output_folder_path=args.output_folder_path,
         output_signaturedefs=args.output_signaturedefs,
+        not_use_onnxsim=args.not_use_onnxsim,
         batch_size=args.batch_size,
         keep_ncw_or_nchw_or_ncdhw_input_names=args.keep_ncw_or_nchw_or_ncdhw_input_names,
         replace_argmax_to_reducemax_and_indicies_is_int64=args.replace_argmax_to_reducemax_and_indicies_is_int64,
@@ -476,6 +545,8 @@ def main():
         replace_acos_to_pseudo_acos=args.replace_acos_to_pseudo_acos,
         replace_leakyrelu_to_pseudo_leakyrelu=args.replace_leakyrelu_to_pseudo_leakyrelu,
         replace_power_to_pseudo_power=args.replace_power_to_pseudo_power,
+        replace_gathernd_to_pseudo_gathernd=args.replace_gathernd_to_pseudo_gathernd,
+        replace_neg_to_pseudo_neg=args.replace_neg_to_pseudo_neg,
         param_replacement_file=args.param_replacement_file,
         mvn_epsilon=args.mvn_epsilon,
         non_verbose=args.non_verbose,
