@@ -5,17 +5,20 @@ np.random.seed(0)
 import tensorflow as tf
 import onnx_graphsurgeon as gs
 from onnx2tf.utils.common_functions import (
+    get_replacement_parameter,
+    replace_parameter,
     get_constant_or_variable,
     convert_axis,
     print_node_info,
     inverted_operation_enable_disable,
     make_tf_node_info,
+    post_process_transpose,
 )
-from onnx2tf.utils.colors import Color
 
 
 @print_node_info
 @inverted_operation_enable_disable
+@get_replacement_parameter
 def make_node(
     *,
     graph_node: gs.Node,
@@ -62,13 +65,14 @@ def make_node(
     if reshape_shape is None:
         reshape_shape = []
 
+    # Ignore
     allowzero = bool(graph_node.attrs.get('allowzero', 0))
-    if allowzero:
-        error_msg = f'' +\
-            f'{Color.RED}ERROR:{Color.RESET} ' +\
-            f'TensorFlow does not support allowzero=1 (True).'
-        print(error_msg)
-        assert not allowzero, error_msg
+
+    # If Reshape's shape contains zeros, get the deformed shape from the output shape
+    if isinstance(reshape_shape, list) and reshape_shape.count(0) > 0:
+        reshape_shape = shape
+    elif isinstance(reshape_shape, np.ndarray) and np.count_nonzero(reshape_shape == 0) > 0:
+        reshape_shape = shape
 
     # Preserving Graph Structure (Dict)
     tf_layers_dict[graph_node_output.name] = {
@@ -106,6 +110,20 @@ def make_node(
     else:
         transposed_reshape_shape = reshape_shape
 
+    # Param replacement
+    transposed_tensor = replace_parameter(
+        value_before_replacement=transposed_tensor,
+        param_target='inputs',
+        param_name=graph_node.inputs[0].name,
+        **kwargs,
+    )
+    transposed_reshape_shape = replace_parameter(
+        value_before_replacement=transposed_reshape_shape,
+        param_target='inputs',
+        param_name=graph_node.inputs[1].name,
+        **kwargs,
+    )
+
     # Reshape
     tf_layers_dict[graph_node_output.name]['tf_node'] = \
         tf.reshape(
@@ -114,6 +132,14 @@ def make_node(
             name=graph_node.name,
         )
 
+    # Post-process transpose
+    tf_layers_dict[graph_node_output.name]['tf_node'] = post_process_transpose(
+        value_before_transpose=tf_layers_dict[graph_node_output.name]['tf_node'],
+        param_target='outputs',
+        param_name=graph_node.outputs[0].name,
+        **kwargs,
+    )
+
     # Generation of Debug Info
     tf_layers_dict[graph_node_output.name]['tf_node_info'] = \
         make_tf_node_info(
