@@ -8,10 +8,14 @@ np.random.seed(0)
 import tensorflow as tf
 import onnx_graphsurgeon as gs
 from onnx2tf.utils.colors import Color
-from typing import Any, List
+from typing import Any, List, Optional
 from functools import wraps
 from collections import namedtuple
 
+from onnx2tf.utils.enums import (
+    TF_DTYPES_TO_NUMPY_DTYPES,
+)
+
 
 def get_replacement_parameter(func):
     @wraps(func)
@@ -30,20 +34,95 @@ def get_replacement_parameter(func):
 
 
 def replace_parameter(
-    value_before_replacement,
-    param_target,
-    param_name,
-    **kwargs,
+    *,
+    value_before_replacement: Any,
+    param_target: str,
+    param_name: str,
+    **kwargs: dict,
 ):
+    """Replace attributes, INPUT constants, and INPUT initializers with the specified values.
+
+    Parameters
+    ----------
+    value_before_replacement: Any
+    param_target: dict
+    param_name: dict
+    **kwargs: dict
+
+    Returns
+    ----------
+    replace_value: Any
+    """
     replace_value = value_before_replacement
-    for op_rep_param in kwargs['op_rep_params']:
+    op_rep_params = kwargs.get('op_rep_params', [])
+    for op_rep_param in op_rep_params:
         if op_rep_param['param_target'] == param_target \
             and op_rep_param['param_name'] == param_name:
             replace_value = op_rep_param.get('values', value_before_replacement)
+            if isinstance(value_before_replacement, np.ndarray):
+                replace_value = np.asarray(
+                    replace_value,
+                    dtype=value_before_replacement.dtype,
+                )
+            elif isinstance(value_before_replacement, list):
+                replace_value = list(replace_value)
+            elif isinstance(value_before_replacement, bool):
+                replace_value = \
+                    bool(replace_value) if isinstance(replace_value, int) and replace_value in [0, 1] else \
+                    bool(int(replace_value)) if isinstance(replace_value, str) and replace_value in ["0", "1"] else \
+                    False if isinstance(replace_value, str) and replace_value.lower() == "false" else \
+                    True if isinstance(replace_value, str) and replace_value.lower() == "True" else \
+                    replace_value
+            elif isinstance(value_before_replacement, int):
+                replace_value = int(replace_value)
+            elif isinstance(value_before_replacement, float):
+                replace_value = float(replace_value)
+            elif isinstance(value_before_replacement, str):
+                replace_value = str(replace_value)
+            elif tf.keras.backend.is_keras_tensor(value_before_replacement):
+                replace_value = np.asarray(
+                    replace_value,
+                    dtype=TF_DTYPES_TO_NUMPY_DTYPES[value_before_replacement.dtype],
+                )
             break
     return replace_value
 
 
+def post_process_transpose(
+    *,
+    value_before_transpose: Any,
+    param_target: str,
+    param_name: str,
+    **kwargs: dict,
+):
+    """Add Transpose as a post-processing step for Reshape OP.
+
+    Parameters
+    ----------
+    value_before_transpose: tf_op
+    param_target: dict
+    param_name: dict
+    **kwargs: dict
+
+    Returns
+    ----------
+    transposed_value: tf_op
+    """
+    transposed_value = value_before_transpose
+    op_rep_params = kwargs.get('op_rep_params', [])
+    for op_rep_param in op_rep_params:
+        if op_rep_param['param_target'] == param_target \
+            and op_rep_param['param_name'] == param_name:
+            transpose_perm = op_rep_param.get('post_process_transpose_perm', None)
+            if transpose_perm is not None:
+                transposed_value = tf.transpose(
+                    a=value_before_transpose,
+                    perm=transpose_perm,
+                )
+            break
+    return transposed_value
+
+
 def make_tf_node_info(**kwargs):
     """Generate information for debug log output.
 
@@ -226,6 +305,7 @@ def inverted_operation_enable_disable(func):
 def get_constant_or_variable(
     const_or_var: Any,
     before_op_output_shape_trans: bool,
+    is_bias: Optional[bool] = False,
 ) -> Any:
     """Get a Numpy constant or gs.Variable from graph_surgeon node.
 
@@ -247,7 +327,7 @@ def get_constant_or_variable(
         if tensor_rank > 2:
             convertion_table = [0] + [i for i in range(2, tensor_rank)] + [1]
             values = values.transpose(convertion_table)
-        elif tensor_rank == 1 and values.size > 2:
+        elif tensor_rank == 1 and values.size > 2 and not is_bias:
             convertion_table = [0] + [i for i in range(2, values.size)] + [1]
             new_values = np.zeros(values.size, dtype=values.dtype)
             for new_idx, idx in enumerate(convertion_table):
@@ -387,6 +467,19 @@ def convert_reverse_axis(
     return converted_axis
 
 
+def channel_transpose(
+    *,
+    const_or_var_1: Any,
+    const_or_var_2: Any,
+):
+    if isinstance(const_or_var_2, np.ndarray) \
+        and len(const_or_var_2.shape) > 1 \
+        and list(const_or_var_2.shape).count(1) == len(const_or_var_2.shape)-1 \
+        and const_or_var_1.shape[-1] == max(const_or_var_2.shape):
+        const_or_var_2 = const_or_var_2.squeeze()
+    return const_or_var_2
+
+
 # https://github.com/onnx/onnx-tensorflow/blob/main/onnx_tf/common/tf_helper.py
 def tf_shape(
     *,
